<?php
/**
 * zogan - PHP template engine for XML
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Haruki Setoyama <haruki@planewave.org>
 * @copyright Copyright &copy; 2003-2004, Haruki SETOYAMA <haruki@planewave.org>
 * @license http://www.opensource.org/licenses/lgpl-license.php LGPL
 * @version 0.2- $Id: zogan.handler.php,v 1.5 2004/10/18 09:17:29 haruki Exp $
 * @link http://phpoot.sourceforge.jp/zogan/
 * @package zogan
 * @subpackage _builder
 */
/**
* some configration
*/
@define('ZOGAN_CONTENT', 'cont');
@define('ZOGAN_ATTRIBUTE', 'attr');
@define('ZOGAN_OMIT', 'omit');
@define('ZOGAN_ALTERNATON', 'alt');
@define('ZOGAN_OMIT_TAG', 'omit-tag');
@define('ZOGAN_FORMAT', 'format');

@define('ZOGAN_NAMESPACE_URI', 'http://zogan.planewave.org/');  // XML namespace URI for Zogan.
@define('ZOGAN_PREFIX', 'var'); // default XML namespace for Zogan. 

// internal use.
define('ZOGAN_VARIABLE_ATTRS', 'vattrs');
define('ZOGAN_NAMESPACE', 'namespace');

/**
* reguler expressions
*/
// elements
@define('ZOGAN_REG_VARIABLE'
    , '[a-zA-Z_\x7f-\xff][a-zA-Z0-9_\x7f-\xff]*');
@define('ZOGAN_REG_FUNC'
    , '[a-zA-Z_\x7f-\xff][a-zA-Z0-9_\x7f-\xff]*');
@define('ZOGAN_OPTION_SPECIAL', '#');
// combined
define('ZOGAN_REG_ARRAY'
    , '(?:'.ZOGAN_REG_VARIABLE.')(?:\/'.ZOGAN_REG_VARIABLE.')*');
define('ZOGAN_REG_VAR'
    , '\{(?:('.ZOGAN_REG_FUNC.')(?:\(((?:[^()]|\\\\\)|\\\\\()*)\))?:)?('.ZOGAN_REG_ARRAY.')\}');
define('ZOGAN_REG_CONT'
    , '/^'.ZOGAN_REG_VAR.'$/D');
define('ZOGAN_REG_CONT_SPECIAL'
    , '/^\{'.ZOGAN_OPTION_SPECIAL.'([a-z]+)(\s+[^}]*)?\}$/D');
define('ZOGAN_REG_ATTR'
    , '/^([^=\s]+)\s*=\s*'.ZOGAN_REG_VAR.'$/D');

/**
* require zogan.elements.php for zogan_element_* classes
*/
require_once ZOGAN_DIR.'/zogan.elements.php';

/**
* handler for XML_HTMLSax
* @package zogan
* @subpackage _builder
*/
class zogan_handler {

    /**
     * template xml text
     * @var string
     * @access private
     */
    var $_template = '';
    /**
     * nest level of $this->elements stack
     * @access private
     */
    var $_level = 0;
    /**
     * stack for zogan_element_* object
     * @access private
     * @var zogan_elements
     */
    var $_elements = array();
    /**
     * Names of special elements. See zogan_element_*
     * @access private
     */
    var $_special_element = array('num' => 1, 'inc' => 1);
    /**
     * @access private
     */
    var $_error = '';

    /**
     * initializze befor parse
     */
    function init($template)
    {
        $this->_template = $template;

        // private prop.
        $this->_level = 0;
        $this->_elements = array();
        $this->_elements[0] = new zogan_element_root;
        $this->_error = '';
    }

    /**
     * handler method for tag opening
     * This method will parse attrs and
     * set zogan_content_tag_$type object in $this->_elements.
     */
    function startHandler(&$parser, $name, $attrs)
    {
        // instructions for template
        // ZOGAN_VARIABLE_ATTRS: the number of valiable attributes
        // ZOGAN_CONTENT: valiable content
        //     array('func' => function name
        //         , 'param' => parameter for function
        //         , 'var'  => array of variable names)
        // or  array('arg' => special content name)
        // ZOGAN_OMIT: true if the tag and its contents must be omited
        //             'first' if omited when first time happning of the tag
        // ZOGAN_OMIT_TAG: true if only the tag must be omited
        // ZOGAN_ALTERNATON: true if the tag is altanative
        // ZOGAN_NAMESPACE: (array) XML namespaces for Zogan 
        $parsed = array();

        // namespace of tag
        $namespace = $this->_elements[$this->_level]->getNamespace();
        foreach ($attrs as $key => $value) {
            if (preg_match('/^xmlns:([^:]+)$/', $key, $match)) {
                if ($value == ZOGAN_NAMESPACE_URI) {
                    $namespace[$match[1]] = $match[1];
                    unset($attrs[$key]);
                }
                elseif (isset($namespace[$match[1]])) {
                    unset($namespace[$match[1]]);
                }
            }
        }
        // parses tags namespaced for Zogan
        foreach ($attrs as $key => $value) {
            if (($div = strpos($key, ':')) !== false
            && $div > 0) {
                $prefix = substr($key, 0, $div); // prefix:tag
                if (isset($namespace[$prefix])) {
                    $localpart = substr($key, $div+1);
                    $attrs[$key] = array($localpart, $value);
                }
            }
        }
        $parsed[ZOGAN_NAMESPACE] = $namespace;

        //
        $element_type = 'static';  // correspond to * in zogan_element_*
        $parsed[ZOGAN_VARIABLE_ATTRS] = 0;
        foreach ($attrs as $key => $value) {
            if (is_array($value)) {
            // attribute as instruction for template
                switch($value[0]){
                case ZOGAN_CONTENT:
                    if (preg_match(ZOGAN_REG_CONT, $value[1], $match)) {
                        $parsed[ZOGAN_CONTENT]
                            = array(
                                'func' => $match[1], 
                                'param' => $match[2],
                                'var' => explode('/', $match[3])
                              );
                        $attrs[$key] = true;
                        $element_type = 'variable';
                    }
                    elseif (preg_match(ZOGAN_REG_CONT_SPECIAL, $value[1], $match)
                    && isset($this->_special_element[$match[1]])) {
                        $parsed[ZOGAN_CONTENT]
                            = array('arg' => isset($match[2]) ? trim($match[2]) : '');
                        $element_type = $match[1];
                        unset($attrs[$key]);
                    }
                    else {
                        unset($attrs[$key]);
                    }
                    break;
                case ZOGAN_OMIT_TAG:
                    if ($value[1] == 'true') {
                        $parsed[ZOGAN_OMIT_TAG] = true;
                    }
                    unset($attrs[$key]);
                    break;
                case ZOGAN_ALTERNATON:
                    if ($value[1] == 'true') {
                        $parsed[ZOGAN_ALTERNATON] = true;
                    }
                    unset($attrs[$key]);
                    break;
                case ZOGAN_OMIT:
                    if ($value[1] == 'true') {
                        $parsed[ZOGAN_OMIT] = true;
                    }
                    elseif ($value[1] == 'first') {
                        $parsed[ZOGAN_OMIT] = 'first';
                    }
                    unset($attrs[$key]);
                    break;
                case ZOGAN_ATTRIBUTE:
                    $parsed[ZOGAN_ATTRIBUTE] = array();
                    $vattrs = explode(',', $value[1]);
                    foreach ($vattrs as $vattr) {
                        if (preg_match(ZOGAN_REG_ATTR, trim($vattr), $match)) {
                            $parsed[ZOGAN_ATTRIBUTE][$match[1]]
                                = array(
                                    'var' => explode('/', $match[4]), 
                                    'func' => $match[2],
                                    'param' => $match[2]
                                  );
                        }
                    }
                    unset($attrs[$key]);
                    break;
                default:
                    unset($attrs[$key]);
                    // TODO: error
                } // switch
            } else {
                if (preg_match(ZOGAN_REG_CONT, $value, $match)) {
                    $attrs[$key]
                        = array(
                            'key' => $key,
                            'var' => explode('/', $match[3]),
                            'func' => $match[1],
                            'param' => $match[2],
                            'default' => $key
                          );
                    $parsed[ZOGAN_VARIABLE_ATTRS]++;
                }
            }
        }

        if (@$parsed[ZOGAN_OMIT] === true) {
            $element_type = 'omit';
        }
        elseif (isset($parsed[ZOGAN_ATTRIBUTE])) {
            foreach ($parsed[ZOGAN_ATTRIBUTE] as $key => $val) {
                if (isset($attrs[$key]) && !is_array($attrs[$key])) {
                    $attrs[$key]
                        = array(
                            'key' => $key,
                            'var' => $val['var'],
                            'func' => $val['func'],
                            'default' => $attrs[$key]
                        );
                }
            }
            unset($parsed[ZOGAN_ATTRIBUTE]);
        }

        //$this->_add_element($element_type);
        // add a element
        $this->_level++;
        unset($this->_elements[$this->_level]);
        $class = 'zogan_element_' . $element_type;
        $this->_elements[$this->_level] = new $class;
        $this->_elements[$this->_level -1]->child[] 
            =& $this->_elements[$this->_level];

        // open the element
        $this->_elements[$this->_level]->open(
              $name
            , $attrs
            , $parsed
            , $this->_elements[$this->_level -1]->info
        );
        $this->_elements[$this->_level]->position
            = $this->_get_current_position($parser);
    }

    /**
     * handler method for tag closing
     */
    function endHandler(&$parser, $close_tag)
    {
        if ($this->_level == 0) {
            $this->_setError($parser, 'not well-formed');
            $parser->stop_parse();
            return;
        }

        if ($this->_elements[$this->_level]->getTagname() != $close_tag) {
            $this->_setError($parser, 'mismatched tag');
            $parser->stop_parse();
            return;
        }

        $empty_tag = false;
        $pos = $parser->get_current_position();
        if ($this->_elements[$this->_level]->position == $pos) {
            $empty_tag = true;
        }
        $this->_elements[$this->_level]->close($empty_tag);
        $this->_level--;

    }

    /**
     * handler method for charactor data
     */
    function cdataHandler(&$parser, $data)
    {
        $lines = preg_split('/(\r\n|\r|\n)/', $data, -1, PREG_SPLIT_DELIM_CAPTURE);
        foreach ($lines as $line) {
            if ($line == "\r\n" || $line == "\n" || $line == "\r") {
                $this->_elements[$this->_level]->child[]
                    = new zogan_content_linefeed($line);
            }
            elseif ($line != '') {
                $this->_elements[$this->_level]->child[]
                    = new zogan_content_cdata($line);
            }
        }
    }

    /**
     * handler method for escape
     */
    function escapeHandler($parser, $data)
    {
        if (preg_match('/^--\s*\#.*--$/', $data)) {
            $this->_elements[$this->_level]->child[]
                = new zogan_content_removed();
        }
        else {
            $this->_elements[$this->_level]->child[]
                = new zogan_content_escape($data);
        }
    }

    /**
     * handler method for pi
     */
    function piHandler($parser, $target, $data)
    {
        $this->_elements[$this->_level]->child[]
         = new zogan_content_pi($target, $data);
    }

    /**
     * handler method for removed content
     */
    function removeHandler($parser, $a1, $a2 = null)
    {
        $this->_elements[$this->_level]->child[]
            = new zogan_content_removed();
    }

    /**
     * get PHPed built template
     */
    function getBuilt()
    {
        return $this->_elements[0]->build();
    }

    /**
     * set error message
     * @access private
     */
    function _setError(&$parser, $msg)
    {
        $pos  = $parser->get_current_position();
        $str  = substr($this->_template, 0 , $pos);
        $str  = str_replace(array("\r\n", "\r"), array("\n", "\n"), $str);
        $line = count(explode("\n", $str));
        $this->_error = $msg ." at XML input line $line ";
    }

    /**
     * get current position in parsing
     * @access private
     */
    function _get_current_position(&$parser)
    {
        return $parser->get_current_position();
    }
}
?>