<?php
/**
 * zogan - PHP template engine for XML
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Haruki Setoyama <haruki@planewave.org>
 * @copyright Copyright &copy; 2003-2004, Haruki SETOYAMA <haruki@planewave.org>
 * @license http://www.opensource.org/licenses/lgpl-license.php LGPL
 * @version 0.2- $Id: zogan.htmlparser.php,v 1.2 2004/10/18 09:17:29 haruki Exp $
 * @link http://phpoot.sourceforge.jp/zogan/
 * @package zogan
 * @subpackage _builder
 */
/**
 * Requires PEAR & XML_HTMLSax 2.1.2, or later.
 */
require_once 'PEAR.php';
require_once 'XML/XML_HTMLSax.php';

/**
 * directory where ZOGAN library is in
 * @ignore
 */
if (!defined('ZOGAN_DIR')) {
    define('ZOGAN_DIR', dirname(__FILE__));
}
/**
* require zogan.handler.php
*/
require_once 'zogan.handler.php';
/**
* handler for XML_HTMLSax
* @package zogan
* @subpackage _builder
*/
class zogan_parser extends PEAR {

    /**
     * zogan_handler & XML_HTMLSax parser
     * @access private
     */
    var $_handler, $_parser;

    /**
     * constructor
     * @access public
     */
    function zogan_parser(&$zogan, $encoding = 'utf-8')
    {
        $vars = array_keys(get_class_vars('PEAR'));
        foreach ($vars as $var) {
            $this->$var =& $zogan->$var;
        }

        $this->_parser = new zogan_XML_HTMLSax();
        $this->_handler = new zogan_handler();

        $this->_parser->set_object($this->_handler);
        $this->_parser->set_element_handler('startHandler', 'endHandler');
        $this->_parser->set_data_handler('cdataHandler');
        $this->_parser->set_pi_handler('piHandler');
        $this->_parser->set_jasp_handler('removeHandler');
        $this->_parser->set_escape_handler('escapeHandler');

        //$this->_parser->set_option('XML_OPTION_LINEFEED_BREAK');
        $this->_parser->set_option('XML_OPTION_FULL_ESCAPES');
    }

    /**
     * initialize before each parse
     * @access public
     */
    function build($template)
    {
        $this->_handler->init($template);
        $this->_parser->parse($template);

        if (!empty($this->_handler->_error)) {
            return $this->raiseError($this->_handler->_error, ZOGAN_ERR_TEMPLATE);
        }
        return $this->_handler->getBuilt();
    }
}

/**
* child class of XML_HTMLSax to add stop_parse() method
* @package zogan
* @subpackage _builder
*/
class zogan_XML_HTMLSax extends XML_HTMLSax {
    function zogan_XML_HTMLSax()
    {
        parent::XML_HTMLSax();
    }

    function stop_parse()
    {
        // this is very dirty.
        $this->state_parser->position = $this->state_parser->length;
    }

}
?>
